==> ba-react-fetch-api-0.7.20.tgz: package/FetchAPI/hoc/list.js <==
// @flow
import _ from 'lodash';
import base from './base';
import {compose, lifecycle, withHandlers} from 'recompose';
import {connect} from 'react-redux';
import type {TFilterItem} from '../typedefs/filter-item';

const
	getCounter = (params: {
		counter: string,
		keyStatePath: string,
		state: Object,
	}) => _.toNumber(
		_.get(params.state, `${params.keyStatePath}.lastResponse.headers.${params.counter}`, 0)
	);

export default (hocParams: {
	baseURL?: string,
	entity: string | (props: Object) => string,
	filters: string | Array<TFilterItem>,
	key: string | (props: Object) => string,
	page: string | number,
	paramsChangeCallback?: Function,
	queryParams?: (props: Object) => Object,
	sorting: string | {column: string, order: string},
	urlParams?: (props: Object) => Object,
}) => (WrappedComponent: any) => compose(
	base(hocParams),

	connect(
		(state, props: {
			faFetched: Object,
			keyStatePath: string,
			resolvedKey: string,
		}) => ({
			faFetched: {
				...props.faFetched,

				[props.resolvedKey]: {
					...props.faFetched[props.resolvedKey],
					filters   : _.isString(hocParams.filters) ? _.get(state, hocParams.filters) : hocParams.filters,
					page      : _.isString(hocParams.page) ? _.get(state, hocParams.page) : hocParams.page,
					pagesCount: getCounter({counter: 'pages-count', keyStatePath: props.keyStatePath, state}),
					sorting   : _.isString(hocParams.sorting) ? _.get(state, hocParams.sorting) : hocParams.sorting,
					totalCount: getCounter({counter: 'total-count', keyStatePath: props.keyStatePath, state}),
				},
			},
		}),
	),

	withHandlers({

		// GET/POST params
		getQueryParams: props => nextProps => {
			const

				/* eslint-disable sort-vars */
				activeProps = (nextProps || props).faFetched[props.resolvedKey],

				filters = _.fromPairs((activeProps.filters || []).map(
					F => ([_.compact([F.field, F.operator]).join('-'), F.compiledValue || F.value])
				)),

				/* eslint-enable sort-vars */
				sorting = _.get(activeProps, 'sorting.column', '');

			let
				request = {};

			if(!_.isEmpty(filters))
				request.filterParams = JSON.stringify(filters);

			if(sorting)
				request.sort = JSON.stringify({[sorting]: _.get(activeProps, 'sorting.order') === 'asc' ? 1 : -1});

			if(activeProps.page)
				request.page = activeProps.page;

			return _.extend(request, activeProps.queryParams);
		},
	}),

	lifecycle({
		componentWillReceiveProps(nextProps) {
			// WARN do not sync if page changed from undefined to 1
			if(nextProps.hasPropsChanged({
				nextProps,
				props     : this.props,
				trackProps: ['filters', 'page', 'sorting', 'queryParams', 'urlParams'],
			})) {
				// WARN Reset sync flag due redux-api cache request
				this.props.faFetched[this.props.resolvedKey].reset('sync');

				this.props.faFetched[this.props.resolvedKey].sync(

					// URL params and query string params must be merged into single object
					// redux-api is able to distinguish URL template params and query string params
					_.extend(
						{},
						nextProps.faFetched[nextProps.resolvedKey].urlParams,
						nextProps.getQueryParams(nextProps),
					),

					// If fetch params changed — call params change callback, ignore page changes
					() => hocParams.paramsChangeCallback &&

						nextProps.hasPropsChanged({
							nextProps,
							props     : this.props,
							trackProps: ['filters', 'sorting', 'queryParams', 'urlParams'],
						}) &&

						!nextProps.hasPropsChanged({nextProps, props: this.props, trackProps: ['page']}) &&
						hocParams.paramsChangeCallback(this.props.dispatch),
				);
			}
		},
	}),
)(WrappedComponent);

==> ba-react-fetch-api-0.7.21.tgz: package/FetchAPI/hoc/list.js <==
// @flow
import _ from 'lodash';
import base from './base';
import {compose, lifecycle, withHandlers} from 'recompose';
import {connect} from 'react-redux';
import type {TFilterItem} from '../flow-typed';

const
	getCounter = (params: {
		counter: string,
		keyStatePath: string,
		state: Object,
	}) => _.toNumber(
		_.get(params.state, `${params.keyStatePath}.lastResponse.headers.${params.counter}`, 0)
	);

export default (hocParams: {
	baseURL?: string,
	entity: string | (props: Object) => string,
	filters: string | Array<TFilterItem>,
	key: string | (props: Object) => string,
	page: string | number,
	paramsChangeCallback?: Function,
	queryParams?: (props: Object) => Object,
	sorting: string | {column: string, order: string},
	urlParams?: (props: Object) => Object,
}) => (WrappedComponent: any) => compose(
	base(hocParams),

	connect(
		(state, props: {
			faFetched: Object,
			keyStatePath: string,
			resolvedKey: string,
		}) => ({
			faFetched: {
				...props.faFetched,

				[props.resolvedKey]: {
					...props.faFetched[props.resolvedKey],
					filters   : _.isString(hocParams.filters) ? _.get(state, hocParams.filters) : hocParams.filters,
					page      : _.isString(hocParams.page) ? _.get(state, hocParams.page) : hocParams.page,
					pagesCount: getCounter({counter: 'pages-count', keyStatePath: props.keyStatePath, state}),
					sorting   : _.isString(hocParams.sorting) ? _.get(state, hocParams.sorting) : hocParams.sorting,
					totalCount: getCounter({counter: 'total-count', keyStatePath: props.keyStatePath, state}),
				},
			},
		}),
	),

	withHandlers({

		// GET/POST params
		getQueryParams: props => nextProps => {
			const

				/* eslint-disable sort-vars */
				activeProps = (nextProps || props).faFetched[props.resolvedKey],

				filters = _.fromPairs((activeProps.filters || []).map(
					F => ([_.compact([F.field, F.operator]).join('-'), F.compiledValue || F.value])
				)),

				/* eslint-enable sort-vars */
				sorting = _.get(activeProps, 'sorting.column', '');

			let
				request = {};

			if(!_.isEmpty(filters))
				request.filterParams = JSON.stringify(filters);

			if(sorting)
				request.sort = JSON.stringify({[sorting]: _.get(activeProps, 'sorting.order') === 'asc' ? 1 : -1});

			if(activeProps.page)
				request.page = activeProps.page;

			return _.extend(request, activeProps.queryParams);
		},
	}),

	lifecycle({
		componentWillReceiveProps(nextProps) {
			// WARN do not sync if page changed from undefined to 1
			if(nextProps.hasPropsChanged({
				nextProps,
				props     : this.props,
				trackProps: ['filters', 'page', 'sorting', 'queryParams', 'urlParams'],
			})) {
				// WARN Reset sync flag due redux-api cache request
				this.props.faFetched[this.props.resolvedKey].reset('sync');

				this.props.faFetched[this.props.resolvedKey].sync(

					// URL params and query string params must be merged into single object
					// redux-api is able to distinguish URL template params and query string params
					_.extend(
						{},
						nextProps.faFetched[nextProps.resolvedKey].urlParams,
						nextProps.getQueryParams(nextProps),
					),

					// If fetch params changed — call params change callback, ignore page changes
					() => hocParams.paramsChangeCallback &&

						nextProps.hasPropsChanged({
							nextProps,
							props     : this.props,
							trackProps: ['filters', 'sorting', 'queryParams', 'urlParams'],
						}) &&

						!nextProps.hasPropsChanged({nextProps, props: this.props, trackProps: ['page']}) &&
						hocParams.paramsChangeCallback(this.props.dispatch),
				);
			}
		},
	}),
)(WrappedComponent);
